package org.ioc.commons.flowcontrol.globalflowcontroller;

import org.ioc.commons.flowcontrol.HasFlowControl;
import org.ioc.commons.flowcontrol.actioncontroller.ActionController;
import org.ioc.commons.flowcontrol.actioncontroller.ActionHandler;
import org.ioc.commons.flowcontrol.actioncontroller.IsExternalAction;
import org.ioc.commons.flowcontrol.common.BindRegistration;
import org.ioc.commons.flowcontrol.eventbus.EventBus;
import org.ioc.commons.flowcontrol.eventbus.EventHandler;
import org.ioc.commons.flowcontrol.eventbus.IsEvent;
import org.ioc.commons.flowcontrol.eventbus.IsEventBus;
import org.ioc.commons.flowcontrol.operationmanager.IsOperation;
import org.ioc.commons.flowcontrol.operationmanager.OperationHandler;
import org.ioc.commons.flowcontrol.operationmanager.OperationManager;
import org.ioc.commons.ui.IsWidget;

/**
 * Implementations of this interface are used to manage the global flow on the
 * entiry application, that is, global events, global actions and global
 * operations which are not related to an specific entity or widget.
 * 
 * @author Jesús Lunar Pérez
 * 
 * @param <E>
 *            Global event type
 * @param <O>
 *            Global operation type
 * @param <A>
 *            Global action type
 */
public interface GlobalFlowController<E extends Enum<? extends IsEvent>, O extends Enum<? extends IsOperation>, A extends Enum<? extends IsExternalAction>>
		extends HasFlowControl<E, O, A> {

	/**
	 * Global event bus
	 * 
	 * @author Jes�s Lunar P�rez
	 * 
	 * @param <E>
	 *            Global event type
	 */
	public interface GlobalEventBus<E extends Enum<? extends IsEvent>> extends EventBus<E>, IsEventBus {
		/**
		 * Binds a global event while the widget passed as parameter is active,
		 * that is, the widget is being shown at this moment.
		 * 
		 * If the widget is not active, the global event fired will not be
		 * managed by this handler until the widget becomes active again.
		 * 
		 * @param whileDisplayIsActive
		 *            Widget which needs to be active for handling the global
		 *            event.
		 * @param event
		 *            Global event
		 * @param handler
		 *            Handler which will process the event in case the widget is
		 *            active
		 * 
		 * @return A bind registration object for managing the new binding
		 *         registered.
		 */
		BindRegistration bindEvent(IsWidget whileDisplayIsActive, E event, EventHandler<E> handler);
	}

	/**
	 * Global operation manager
	 * 
	 * @author Jes�s Lunar P�rez
	 * 
	 * @param <O>
	 *            Global operation type
	 */
	public interface GlobalOperationManager<O extends Enum<? extends IsOperation>> extends OperationManager<O> {
		/**
		 * Binds a global operation while the widget passed as parameter is
		 * active, that is, the widget is being shown at this moment.
		 * 
		 * If the widget is not active, the global operation will not be managed
		 * by this handler until the widget becomes active again.
		 * 
		 * @param whileDisplayIsActive
		 *            Widget which needs to be active for handling the global
		 *            operation.
		 * @param operation
		 *            Global operation
		 * @param handler
		 *            Handler which will process the operation in case the
		 *            widget is active
		 * @return A bind registration object for managing the new binding
		 *         registered.
		 */
		BindRegistration bindOperation(IsWidget whileDisplayIsActive, O operation, OperationHandler<O> handler);

		/**
		 * Binds a global operation and automatically unbinds it when it
		 * finishes, while the widget passed as parameter is active, that is,
		 * the widget is being shown at this moment.
		 * 
		 * If the widget is not active, the global event fired will not be
		 * managed by this handler until the widget becomes active again.
		 * 
		 * @param whileDisplayIsActive
		 *            Widget which needs to be active for handling the global
		 *            operation.
		 * @param operation
		 *            Global operation
		 * @param handler
		 *            Handler which will process the operation in case the
		 *            widget is active
		 * @return A bind registration object for managing the new binding
		 *         registered.
		 */
		BindRegistration bindOperationOnce(IsWidget whileDisplayIsActive, O operation, OperationHandler<O> handler);

		/**
		 * Bind for all operations, while the widget passed as parameter is
		 * active, that is, the widget is being shown at this moment. The
		 * handler will be fired when first operation begins running and when
		 * last operation ends.
		 * 
		 * @param whileDisplayIsActive
		 *            Widget which needs to be active for handling the global
		 *            operation.
		 * @param handler
		 *            Handler which will process the operation in case the
		 *            widget is active
		 * @return A bind registration object for managing the new binding
		 *         registered.
		 */
		BindRegistration bindOperationsRunning(IsWidget whileDisplayIsActive, OperationHandler<O> handler);
	}

	/**
	 * Global action manager
	 * 
	 * @author Jes�s Lunar P�rez
	 * 
	 * @param <A>
	 *            Action type
	 */
	public interface GlobalActionController<A extends Enum<? extends IsExternalAction>> extends ActionController<A> {
		/**
		 * Binds a global action while the widget passed as parameter is active,
		 * that is, the widget is being shown at this moment.
		 * 
		 * If the widget is not active, the global action invoked will not be
		 * managed by this handler until the widget becomes active again.
		 * 
		 * @param whileDisplayIsActive
		 *            Widget which needs to be active for handling the global
		 *            action.
		 * @param action
		 *            Global action
		 * @param handler
		 *            Handler which will process the action in case the widget
		 *            is active
		 * @return A bind registration object for managing the new binding
		 *         registered.
		 */
		BindRegistration bindAction(IsWidget whileDisplayIsActive, A action, ActionHandler<A> handler);
	}

	/**
	 * @return Global event bus
	 */
	GlobalEventBus<E> eventBus();

	/**
	 * @return Global operation manager
	 */
	GlobalOperationManager<O> operationManager();

	/**
	 * @return Global action controller
	 */
	GlobalActionController<A> actionController();
}
