package org.ioc.commons.integration.dataaccess.dao;

import java.util.List;

import org.ioc.commons.flowcontrol.common.BindRegistration;
import org.ioc.commons.flowcontrol.operationmanager.IsOperation;
import org.ioc.commons.flowcontrol.operationmanager.OperationManagerTrigger;

/**
 * Binder for entity changes
 * 
 * @author Jesús Lunar Pérez
 * 
 * @param <T>
 *            Entity type
 * @param <I>
 *            Entity identifier type
 */
public interface SubscriptionChangesBinder<T, I> {
	/**
	 * Binds for changes on <T> entities
	 * 
	 * @param handler
	 *            Handler which will process the some operations when changes
	 *            occurred.
	 * @return A bind registration object for managing the new binding
	 *         registered.
	 */
	BindRegistration bindChanges(SubscriptionChangeHandler<I> handler);

	/**
	 * Binds only the first change on some <T> entity and it unbids
	 * automatically.
	 * 
	 * @param handler
	 *            Handler which will process the some operations when changes
	 *            occurred.
	 * @return A bind registration object for managing the new binding
	 *         registered.
	 */
	BindRegistration bindChangesOnce(SubscriptionChangeHandler<I> handler);

	/**
	 * Update a list of entities when a change has been made on an entity.
	 * 
	 * A <T> entity will be added when anentity is persisted. A <T> entity will
	 * be replaced in case the list constains it when is updated. A <T> entity
	 * will be removed from the lists in case the list constains it when is
	 * deleted.
	 * 
	 * @param collection
	 *            List of entities
	 * @param propertyRefs
	 *            The minimum required properties (accesible attributes of the T
	 *            class through getters) to be filled in the T entity before
	 *            using in the list.
	 * @return A bind registration object for managing the new binding
	 *         registered.
	 */
	BindRegistration updateList(List<T> collection, String... propertyRefs);

	/**
	 * It performs the same operation as {@link #updateList(List, String...)}
	 * but just once and then it unbinds automatically.
	 * 
	 * @param collection
	 *            List of entities
	 * @param propertyRefs
	 *            The minimum required properties (accesible attributes of the T
	 *            class through getters) to be filled in the T entity before
	 *            using in the list.
	 * @return A bind registration object for managing the new binding
	 *         registered.
	 */
	BindRegistration updateListOnce(List<T> collection, String... propertyRefs);

	/**
	 * It performs the same operation as {@link #updateList(List, String...)}
	 * but managing an operation during the updation.
	 * 
	 * @param operation
	 *            Operation
	 * @param trigger
	 *            Trigger for managing
	 * @param collection
	 *            List of entities
	 * @param propertyRefs
	 *            The minimum required properties (accesible attributes of the T
	 *            class through getters) to be filled in the T entity before
	 *            using in the list.
	 * @return A bind registration object for managing the new binding
	 *         registered.
	 */
	<O extends Enum<? extends IsOperation>> BindRegistration updateList(O operation,
			OperationManagerTrigger<O> trigger, List<T> collection, String... propertyRefs);

	/**
	 * It performs the same operation as
	 * {@link #updateList(Enum, OperationManagerTrigger, List, String...)} but
	 * just once and then it unbinds automatically.
	 * 
	 * @param operation
	 *            Operation
	 * @param trigger
	 *            Trigger for managing
	 * @param collection
	 *            List of entities
	 * @param propertyRefs
	 *            The minimum required properties (accesible attributes of the T
	 *            class through getters) to be filled in the T entity before
	 *            using in the list.
	 * @return A bind registration object for managing the new binding
	 *         registered.
	 */
	<O extends Enum<? extends IsOperation>> BindRegistration updateListOnce(O operation,
			OperationManagerTrigger<O> trigger, List<T> collection, String... propertyRefs);

}
